#' Grab description of available data for one lake
#'
#' @param URL URL of the lake web page
#'
#' @return List corresponding to the JSON data available in the web page
#' @export
#'
#' @import httr
#' @importFrom jsonlite fromJSON
#' @importFrom stringi stri_match_all_regex
#' @importFrom textutils HTMLdecode
#'
#' @examples
#' SGL_GrabVariableListLake("https://seinegrandslacs.fr/quatre-lacs-reservoirs/lac-reservoir-seine")
#'
SGL_GrabVariableListLake <- function(URL) {
  r <- httr::GET(URL)
  sJSON <- stringi::stri_match_all_regex(httr::content(r, "text"), "data-locations='(.+)'")[[1]][1,2]
  jsonlite::fromJSON(gsub("\\\\", "", textutils::HTMLdecode(sJSON)))
}


#' Grab description of available data for several lakes
#'
#' @param URL Base URL without the lake name (See default value).
#' @param lakes lake names in the URL (See default value).
#'
#' @return Dataframe with description of all available data
#' @export
#'
#' @examples
#' SGL_GrabVariableListAll()
#'
SGL_GrabVariableListAll <- function(
  URL = "https://seinegrandslacs.fr/quatre-lacs-reservoirs/lac-reservoir-",
  lakes = c("marne", "seine", "aube", "de-panneciere")
) {
  do.call(rbind, lapply(paste0(URL, lakes), SGL_GrabVariableListLake))
}


#' Grab data for one measuring point
#'
#' @param URL Complete URL of the data source.
#'
#' @return A dataframe containing stata time series with 2 columns: `datetime` ([character] format "YYYY-MM-DDThh:mm") and [numeric] `value`.
#' @export
#'
#' @import httr
#'
#' @examples
#' varlist <- SGL_GrabVariableListLake(
#'   "https://seinegrandslacs.fr/quatre-lacs-reservoirs/lac-reservoir-seine"
#' )
#' SGL_GrabVariable(varlist$link[1])
#'
SGL_GrabVariable <- function(URL) {
  r <- httr::GET(URL)
  lData <- httr::content(r, "parsed")
  lData$title <- NULL
  lData$param <- NULL
  dfData <- as.data.frame(do.call("rbind", lData))
  dfData$datetime <- unlist(dfData$datetime)
  dfData$value <- as.numeric(dfData$value)
  return(dfData)
}


#' Grab all available data on SGL web site for a list of lakes
#'
#' @param URL Base URL without the lake name (See default value).
#' @param lakes lake names in the URL (See default value).
#'
#' @return A list with 2 items:
#' - a key "V" containing a dataframe with the detail of each measuring point.
#' - a key "Data" containing a dataframe with columns `id`, `datetime` and `value`.
#' @export
#'
#' @importFrom dplyr bind_rows
#'
#' @examples
#' SGL_GrabAllVariables()
#'
SGL_GrabAllVariables <- function(
  URL = "https://seinegrandslacs.fr/quatre-lacs-reservoirs/lac-reservoir-",
  lakes = c("marne", "seine", "aube", "de-panneciere")
) {
  lV <- SGL_GrabVariableListAll(URL, lakes)
  lData <- lapply(lV$link, SGL_GrabVariable)
  names(lData) <- lV$id
  dfData <- dplyr::bind_rows(lData, .id = "id")
  return(list(V = lV, Data = dfData))
}
